#!/usr/bin/env python
"""TableIO.py
A module for reading ASCII tables into NumPy arrays (and lists). 

Written by Michael A. Miller <miller5@uiuc.edu>
Time-stamp: <1998-06-04 11:52:41 miller> """

__version__ = "$Revision: 1.4 $"
# $Source: /home/miller/python/TableIO/RCS/TableIO.py,v $
   
try:
	import numpy as Num
except ImportError:
	import Numeric as Num
import _tableio

def TableToArray(t):
    a = Num.array(t['data'])
    a.shape = (t['rows'],t['columns'])
    return a

def readTable(fname, commentChars):
    t = _tableio.readTable(fname, commentChars)
    return t

def readTableAsArray(fname, commentChars):
    t = _tableio.readTable(fname, commentChars)
    a = TableToArray(t)
    return a

def readColumns(fname, commentChars, columns='all'):
    t = _tableio.readTable(fname, commentChars)
    return extractColumns(t, columns)

def extractColumns(t, columns='all'):
    a = Num.array(t['data'])
    a.shape = (t['rows'],t['columns'])
    if columns == 'all':
        data = []
        for c in range(t['columns']):
            data.append(Num.ravel(Num.take(a, [c], 1)))
    else:
        if type(columns) == type([]):
            data = []
            for c in columns:
                data.append(Num.ravel(Num.take(a, [c], 1)))
                
    return data

def writeArray(fname, data ):
    t = {}
    t['filename'] = fname
    t['data'] = Num.ravel(data).tolist()
    t['rows'] = data.shape[0]
    t['columns'] = data.shape[1]
    status = writeTable( t )
    return status

def writeTable(t):
    status = _tableio.writeTable(t['filename'], t['data'], t['rows'], t['columns'])
    return status

def TableFromColumns(list):
    t = {}
    for l in map(len,list):
        if l != len(list[0]):
            print 'Columns must all have the same length', map(len,list)
    else:
        t['filename'] = ''
        t['rows'] = len(list[0])
        t['columns'] = len(list)
        t['data'] = Num.ravel(Num.transpose(Num.array(list))).tolist()
    return t

def test():
    # Make a table and write it to a file:
    fname = 'TableIO.test-data'
    print 'Writing a table to', fname
    t = {}
    t['columns'] = 3
    t['rows'] = 5
    t['filename'] = fname
    t['data'] = [ 4,  4,  1.3,
                  8,  8,  1.7,
                  9,  9,  1.8,
                  10, 10, 1.9,
                  11, 11, 0.000000001,
                  11, 11, 1e10 ]
    writeTable(t)

    # Read it back in:
    print 'Reading it back as columns'   
    [x,y,z] = readColumns(fname, '#!')
    [a, b] = readColumns(fname, '#!', [0,2])
    print 'x =', x
    print 'y =', y
    print 'z =', z

    # Creat a table from columns and write it out
    print 'Creating a table from columns and writing it'
    u = TableFromColumns([x,y,z,x*x,y,z-y])
    u['filename'] = fname + '2'
    writeTable(u)
    print 'Table is', u

    print 'Read it back as a table:'
    t = readTable(fname, '#!')
    print t

    print '... and as an array'
    a = readTableAsArray(fname, '!#')
    print type(a), a.shape
    print a

    print 'Writing array to file', fname+'3'
    writeArray(fname+'3', a)

    print 'Testing comments'
    f = open(fname+'4','w')
    f.writelines(['# A comment line\n',
                  '1 2 3 4 5\n',
                  '1 2 3 4 5\n',
                  '# A comment line\n',
                  '1 2 3 4 5\n'])
    f.close()
    t = readTable(fname+'4','#')

    f = open(fname+'5','w')
    f.writelines(['# A comment line\n',
                  'X Also comment line\n',
                  '! Also comment line\n',
                  '? Also comment line\n',
                  '1 2 3 4 5\n',
                  '1 2 3 4 5\n',
                  '# A comment line\n',
                  '1 2 3 4 5\n'])
    f.close()
    t = readTable(fname+'5','#X!?')

    
if __name__ == '__main__':
    test()
